# frozen_string_literal: true

module Ephemeron
  module ModelAddons
    extend ActiveSupport::Concern

    included do
      after_find lambda { |model|
        Store.add model, log: true
      }

      after_validation lambda { |model|
        Logger.log model, :used, soft: true
      }

      before_save lambda { |model|
        Validators::BeforeSave.call model
      }

      after_create lambda { |model|
        Store::Update.call model
      }

      after_save lambda { |model|
        Logger.log model, :saved
      }

      after_destroy lambda { |model|
        Ephemeron::Store.destroy model
      }
    end
  end
end
