# frozen_string_literal: true

module Ephemeron
  class Logger < ThreadStore
    attribute :ts_logs

    PROXY_EVENTS = %i[proxy_start proxy_end].freeze
    VALID_EVENTS = %i[fetched fetch_allowed saved save_allowed skip_save used] + PROXY_EVENTS

    def logs
      self.ts_logs ||= []
    end

    def has?(model, event)
      validate model, event
      logs.include? Convert.call(model, event)
    end

    def log(model, event, opts = {})
      validate model, event
      logs << Convert.call(model, event)
    rescue LoggerError, StoreError => e
      raise e unless opts[:soft]
    end

    def reset
      self.ts_logs = []
    end

    private

    def validate(model, event)
      raise(LoggerError, "event (#{event}) is invalid") if VALID_EVENTS.exclude?(event)
      return true if validate_proxy_events model, event

      raise(StoreError, "can't #{event_name_for_validate event} for nil") if model.nil?
      return true if Store.has?(model)
      return true if event == :saved

      raise StoreError, "can't #{event_name_for_validate event} " \
                        "if #{Store::Convert.call model} is not in store"
    end

    def validate_proxy_events(model, event)
      return false if PROXY_EVENTS.exclude?(event)
      return true if model.nil?

      raise LoggerError, 'Proxy events can be logged for nil only'
    end

    def event_name_for_validate(event)
      if event == :used
        'mark as used'
      else
        'log'
      end
    end
  end
end
